// Generated by CoffeeScript 1.4.0
var And, JOIN_TYPES, Join, Node, SUDQuery, Select, SelectQuery, fluidize, getAlias, sqlFunction, toRelation, _ref,
  __hasProp = {}.hasOwnProperty,
  __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  __slice = [].slice;

fluidize = require('../fluid');

SUDQuery = require('./sud');

_ref = require('../nodes'), Node = _ref.Node, getAlias = _ref.getAlias, Select = _ref.Select, And = _ref.And, Join = _ref.Join, toRelation = _ref.toRelation, sqlFunction = _ref.sqlFunction, JOIN_TYPES = _ref.JOIN_TYPES;

module.exports = SelectQuery = (function(_super) {

  __extends(SelectQuery, _super);

  /*
    Adds a number of SELECT-specific methods to :class:`queries/sud::SUDQuery`,
    such as `fields` and `groupBy`
  */


  function SelectQuery() {
    return SelectQuery.__super__.constructor.apply(this, arguments);
  }

  SelectQuery.rootNode = Select;

  SelectQuery.prototype.fields = function() {
    var alias, f, fields, proj, rel, _i, _len,
      _this = this;
    fields = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
    /*
        Adds one or more fields to the query. Fields can be strings (in which case
        they will be passed to :meth:`queries/sud::SUDQuery.project`) or pre-
        constructed nodes. (Such as those returned by ``project``).
    
        If no fields are given, clears all fields from the currently focused table.
    
        To alias a field, use an object with a single key where the key is the alias
        name and the value is a string or node::
    
          q.fields({employee_name: 'employees.name'})
    */

    if (fields.length === 0) {
      rel = this.q.relations.active;
      this.q.projections.prune(function(p) {
        return p.rel() === rel;
      });
      return;
    }
    proj = function(o) {
      if (o instanceof Node) {
        return o;
      } else {
        return _this.project(o);
      }
    };
    for (_i = 0, _len = fields.length; _i < _len; _i++) {
      f = fields[_i];
      if (alias = getAlias(f)) {
        f = f[alias];
        this.q.projections.addNode(proj(f).as(alias));
      } else {
        this.q.projections.addNode(proj(f));
      }
    }
    return null;
  };

  SelectQuery.prototype.agg = function() {
    var alias, fields, fun;
    fun = arguments[0], fields = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
    /*
        Adds one or more aggregated fields to the query
    
        :param fun: name of SQL aggregation function.
        :param fields: Fields to be projected from the current table and passed
          as arguments to ``fun``
    
        Example::
    
          select('t1', function (q) { q.agg('count', q.p('id')) }) # SELECT count(id) FROM t1
          select('t1', function (q) { q.agg({counter: 'count'}, q.p('id')) }) # SELECT count(id) AS "counter" FROM t1
    */

    if (alias = getAlias(fun)) {
      return this.q.projections.addNode(sqlFunction(fun[alias], fields).as(alias));
    } else {
      return this.q.projections.addNode(sqlFunction(fun, fields));
    }
  };

  SelectQuery.prototype.distinct = function(bool) {
    /*
        Make this query DISTINCT on *all* fields.
    */
    return this.q.distinct.enable = bool;
  };

  SelectQuery.prototype.join = function(table, opts) {
    var join, rel, type;
    if (opts == null) {
      opts = {};
    }
    /*
        Join another table to the query.
    
        :param table: A table name, or alias literal. An error will be thrown if
          the table/alias name is not unique. See :func:`nodes::toRelation` for
          more information on the many things ``table`` could be.
        :param opts.on:
          An object literal expressing join conditions. See
          :meth:`queries/select::SelectQuery::where` for more.
        :param opts.type: A join type constant (e.g. INNER, OUTER)
        :param opts.fields: A list of fields to be projected from the newly joined table.
    */

    rel = toRelation(table);
    if (this.q.relations.get(rel.ref(), false)) {
      throw new Error("Table/alias " + (rel.ref()) + " is not unique!");
    }
    type = opts.type || JOIN_TYPES.INNER;
    if (!(type instanceof JOIN_TYPES.INNER.constructor)) {
      throw new Error("Invalid join type " + type + ", try the constant types exported in the base module (e.g. INNER).");
    }
    join = new Join(type, rel);
    this.q.relations.addNode(join);
    if (opts.on) {
      join.on(new And(this._makeClauses(opts.on)));
    }
    if (opts.fields != null) {
      return this.fields.apply(this, opts.fields);
    }
  };

  SelectQuery.prototype.ensureJoin = function(table, opts) {
    var rel;
    if (opts == null) {
      opts = {};
    }
    /*
        The same as :meth:`join`, but will only join ``tbl`` if it is **not**
        joined already.
    */

    rel = toRelation(table);
    if (!this.q.relations.get(rel.ref(), false)) {
      return this.join(rel, opts);
    }
  };

  SelectQuery.prototype.focus = function(alias) {
    /*
        Make a different table "focused", this will use that table as the default
        for the ``fields``, ``order`` and ``where`` methods.
    
        :param alias: The table/alias name to focus. If the table or alias is not
          already part of the query an error will be thrown.
    */
    return this.q.relations["switch"](alias);
  };

  SelectQuery.prototype.groupBy = function() {
    var field, fields, _i, _len;
    fields = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
    /* Add a GROUP BY to the query.
    */

    for (_i = 0, _len = fields.length; _i < _len; _i++) {
      field = fields[_i];
      this.q.groupBy.addNode(this.project(field));
    }
    return null;
  };

  SelectQuery.prototype.having = function(constraint) {
    /*
        This method works similarly to :meth:`queries/sud::SUDQuery.where`, but the
        constraints are added the `HAVING` portion of a SQL clause.
    */

    var node, _i, _len, _ref1, _results;
    _ref1 = this._makeClauses(constraint);
    _results = [];
    for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
      node = _ref1[_i];
      _results.push(this.q.having.addNode(node));
    }
    return _results;
  };

  return SelectQuery;

})(SUDQuery);

fluidize(SelectQuery, 'distinct', 'fields', 'agg', 'join', 'ensureJoin', 'focus', 'groupBy', 'having');

SelectQuery.prototype.field = SelectQuery.prototype.fields;
